<?php
declare(strict_types=1);

namespace Inti\App;

use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\Stratigility\Middleware\ErrorHandler;
use Mezzio\Application;
use Mezzio\Handler\NotFoundHandler;
use Mezzio\Helper\ServerUrlMiddleware;
use Mezzio\Helper\UrlHelperMiddleware;
use Mezzio\Router\Middleware\DispatchMiddleware;
use Mezzio\Router\Middleware\ImplicitHeadMiddleware;
use Mezzio\Router\Middleware\ImplicitOptionsMiddleware;
use Mezzio\Router\Middleware\MethodNotAllowedMiddleware;
use Mezzio\Router\Middleware\RouteMiddleware;

/**
 * Class ConfigProvider
 *
 * @package Inti\App
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-app
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return (new ConfigAggregator([
            \Mezzio\ConfigProvider::class,
            \Mezzio\Helper\ConfigProvider::class,
            \Mezzio\Router\ConfigProvider::class,
            \Mezzio\Router\FastRouteRouter\ConfigProvider::class,

            new ArrayProvider([
                Application::class => [
                    'inject_default_pipeline' => true,

                    'default_pipeline' => [[
                        'middleware' => ErrorHandler::class,
                        'priority' => 100,
                    ], [
                        'middleware' => ServerUrlMiddleware::class,
                        'priority' => 99,
                    ], [
                        'middleware' => RouteMiddleware::class,
                        'priority' => 75,
                    ], [
                        'middleware' => ImplicitHeadMiddleware::class,
                        'priority' => 75,
                    ], [
                        'middleware' => ImplicitOptionsMiddleware::class,
                        'priority' => 75,
                    ], [
                        'middleware' => MethodNotAllowedMiddleware::class,
                        'priority' => 75,
                    ], [
                        'middleware' => UrlHelperMiddleware::class,
                        'priority' => 75,
                    ], [
                        'middleware' => DispatchMiddleware::class,
                        'priority' => 10,
                    ], [
                        'middleware' => NotFoundHandler::class,
                        'priority' => -10,
                    ]],
                ],

                'dependencies' => [
                    'delegators' => [
                        Application::class => [
                            Container\ApplicationDelegator::class,
                        ],
                    ],
                ],
            ]),
        ]))->getMergedConfig();
    }
}
