<?php
declare(strict_types=1);

namespace Inti\App\Container;

use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\DelegatorFactoryInterface;
use Laminas\Stdlib\ArrayUtils;
use Mezzio\Application;
use Mezzio\Container\ApplicationConfigInjectionDelegator;

/**
 * Class ApplicationDelegator
 *
 * @package Inti\App\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-app
 */
final class ApplicationDelegator implements DelegatorFactoryInterface
{
    public function __invoke(ContainerInterface $container, $name, callable $callback, array $options = null): Application
    {
        $config = $container->has('config') ? $container->get('config') : [];

        if ($config[Application::class]['inject_default_pipeline'] ?? false) {
            $config['middleware_pipeline'] = ArrayUtils::merge(
                $config[Application::class]['default_pipeline'] ?? [],
                $config['middleware_pipeline'] ?? []
            );
        }

        /** @var Application $application */
        $application = $callback();

        ApplicationConfigInjectionDelegator::injectPipelineFromConfig($application, $config);
        ApplicationConfigInjectionDelegator::injectRoutesFromConfig($application, $config);

        return $application;
    }
}
