<?php
/** @noinspection PhpParamsInspection */
/** @noinspection PhpUndefinedMethodInspection */
declare(strict_types=1);

namespace IntiTest\App\Container;

use Inti\Test\TestCase\ApplicationTestCase;
use Laminas\Stratigility\Exception\EmptyPipelineException;
use Mezzio\Application;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Class ApplicationDelegatorTest
 *
 * @package IntiTest\App\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-app
 */
final class ApplicationDelegatorTest extends ApplicationTestCase
{
    use ProphecyTrait;

    public function testApplicationWillInjectDefaultPipelineByDefault()
    {
       $response = $this->runApplication();
        $this->assertEquals(404, $response->getStatusCode());
    }

    public function testApplicationWillNotInjectDefaultPipelineIfDisabled()
    {
        $application = $this->createApplication([
            Application::class => [
                'inject_default_pipeline' => false,
            ],
        ]);

        $this->expectException(EmptyPipelineException::class);

        $application->run();
    }

    public function testCanPipeMiddlewareViaConfig()
    {
        /** @var ResponseInterface|ObjectProphecy $expected */
        $expected = $this->prophesize(ResponseInterface::class)->reveal();

        /** @var MiddlewareInterface|ObjectProphecy $middleware */
        $middleware = $this->prophesize(MiddlewareInterface::class);
        $middleware->process(Argument::any(), Argument::any())
            ->shouldBeCalledOnce()
            ->willReturn($expected);

        $response = $this->runApplication([
            'middleware_pipeline' => [[
                'middleware' => $middleware->reveal(),
                'priority' => 50,
            ]],
        ]);

        $this->assertSame($expected, $response);
    }

    public function testCanRouteHandlerViaConfig()
    {
        /** @var ResponseInterface|ObjectProphecy $expected */
        $expected = $this->prophesize(ResponseInterface::class)->reveal();

        /** @var RequestHandlerInterface|ObjectProphecy $handler */
        $handler = $this->prophesize(RequestHandlerInterface::class);
        $handler->handle(Argument::any())
            ->shouldBeCalledOnce()
            ->willReturn($expected);

        $response = $this->runApplication([
            'routes' => [[
                'path' => '/test-route',
                'middleware' => $handler->reveal(),
            ]]
        ], '/test-route');

        $this->assertSame($expected, $response);
    }
}
